/*
** send.c -- send a composed message
**
** This code is Copyright (c) 2002, by the authors of nmh.  See the
** COPYRIGHT file in the root directory of the nmh distribution for
** complete copyright information.
*/

#include <h/mh.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>


static struct swit switches[] = {
#define ALIASW  0
	{ "alias aliasfile", 0 },
#define DEBUGSW  1
	{ "debug", -5 },
#define FILTSW  2
	{ "filter filterfile", 0 },
#define NFILTSW  3
	{ "nofilter", 0 },
#define FRMTSW  4
	{ "format", 0 },
#define NFRMTSW  5
	{ "noformat", 0 },
#define FORWSW  6
	{ "forward", 0 },
#define NFORWSW  7
	{ "noforward", 0 },
#define PUSHSW  8
	{ "push", 0 },
#define NPUSHSW  9
	{ "nopush", 0 },
#define UNIQSW  10
	{ "unique", -6 },
#define NUNIQSW  11
	{ "nounique", -8 },
#define VERBSW  12
	{ "verbose", 0 },
#define NVERBSW  13
	{ "noverbose", 0 },
#define WATCSW  14
	{ "watch", 0 },
#define NWATCSW  15
	{ "nowatch", 0 },
#define WIDTHSW  16
	{ "width columns", 0 },
#define VERSIONSW  17
	{ "version", 0 },
#define HELPSW  18
	{ "help", 0 },
	{ NULL, 0 }
};

extern int debugsw;  /* from sendsbr.c */
extern int forwsw;
extern int inplace;
extern int pushsw;
extern int unique;
extern int verbsw;

extern char *altmsg;  /*  .. */
extern char *annotext;
extern char *distfile;


int
main(int argc, char **argv)
{
	int msgp = 0, distsw = 0, vecp = 1;
	int msgnum, status;
	char *cp, *maildir = NULL;
	char buf[BUFSIZ], **ap, **argp, **arguments;
	char *msgs[MAXARGS], *vec[MAXARGS];
	struct msgs *mp;
	struct stat st;

#ifdef LOCALE
	setlocale(LC_ALL, "");
#endif
	invo_name = mhbasename(argv[0]);

	/* read user profile/context */
	context_read();

	arguments = getarguments(invo_name, argc, argv, 1);
	argp = arguments;

	vec[vecp++] = "-library";
	vec[vecp++] = getcpy(toabsdir("+"));

	while ((cp = *argp++)) {
		if (*cp == '-') {
			switch (smatch(++cp, switches)) {
			case AMBIGSW:
				ambigsw(cp, switches);
				done(1);
			case UNKWNSW:
				adios(NULL, "-%s unknown\n", cp);

			case HELPSW:
				snprintf(buf, sizeof(buf),
						"%s [file] [switches]",
						invo_name);
				print_help(buf, switches, 1);
				done(1);
			case VERSIONSW:
				print_version(invo_name);
				done(1);

			case PUSHSW:
				pushsw++;
				continue;
			case NPUSHSW:
				pushsw = 0;
				continue;

			case UNIQSW:
				unique++;
				continue;
			case NUNIQSW:
				unique = 0;
				continue;

			case FORWSW:
				forwsw++;
				continue;
			case NFORWSW:
				forwsw = 0;
				continue;

			case VERBSW:
				verbsw++;
				vec[vecp++] = --cp;
				continue;
			case NVERBSW:
				verbsw = 0;
				vec[vecp++] = --cp;
				continue;

			case DEBUGSW:
				debugsw++;  /* fall */
			case NFILTSW:
			case FRMTSW:
			case NFRMTSW:
			case WATCSW:
			case NWATCSW:
				vec[vecp++] = --cp;
				continue;

			case ALIASW:
			case FILTSW:
			case WIDTHSW:
				vec[vecp++] = --cp;
				if (!(cp = *argp++) || *cp == '-')
					adios(NULL, "missing argument to %s",
							argp[-2]);
				vec[vecp++] = cp;
				continue;

			}
		} else {
			msgs[msgp++] = cp;
		}
	}

	/*
	** check for "Aliasfile:" profile entry
	*/
	if ((cp = context_find("Aliasfile"))) {
		char *dp = NULL;

		for (ap = brkstring(dp = getcpy(cp), " ", "\n"); ap && *ap;
				ap++) {
			vec[vecp++] = "-alias";
			vec[vecp++] = *ap;
		}
	}

	if (!msgp)
		msgs[msgp++] = seq_cur;
	maildir = toabsdir(draftfolder);

	if (chdir(maildir) == NOTOK)
		adios(maildir, "unable to change directory to");

	/* read folder and create message structure */
	if (!(mp = folder_read(draftfolder)))
		adios(NULL, "unable to read draft folder %s", draftfolder);

	/* check for empty folder */
	if (mp->nummsg == 0)
		adios(NULL, "no messages in draft folder %s", draftfolder);

	/* parse all the message ranges/sequences and set SELECTED */
	for (msgnum = 0; msgnum < msgp; msgnum++)
		if (!m_convert(mp, msgs[msgnum]))
			done(1);
	seq_setprev(mp);  /* set the previous-sequence */

	for (msgp = 0, msgnum = mp->lowsel; msgnum <= mp->hghsel; msgnum++) {
		if (is_selected(mp, msgnum)) {
			msgs[msgp++] = getcpy(m_name(msgnum));
			unset_exists(mp, msgnum);
		}
	}

	mp->msgflags |= SEQMOD;
	seq_save(mp);

	if (!(cp = getenv("SIGNATURE")) || !*cp)
		if ((cp = context_find("signature")) && *cp)
			m_putenv("SIGNATURE", cp);

	for (msgnum = 0; msgnum < msgp; msgnum++)
		if (stat(msgs[msgnum], &st) == NOTOK)
			adios(msgs[msgnum], "unable to stat draft file");

	if ((annotext = getenv("mhannotate")) == NULL || *annotext == 0)
		annotext = NULL;
	if (annotext && ((cp = getenv("mhinplace")) != NULL && *cp != 0))
		inplace = atoi(cp);
	if ((altmsg = getenv("mhaltmsg")) == NULL || *altmsg == 0)
		altmsg = NULL;  /* used by dist interface - see below */

	if ((cp = getenv("mhdist")) && *cp && (distsw = atoi (cp)) && altmsg) {
		vec[vecp++] = "-dist";
		distfile = getcpy(m_mktemp2(altmsg, invo_name, NULL, NULL));
		if (link(altmsg, distfile) == NOTOK) {
			if (errno != EXDEV
#ifdef EISREMOTE
					&& errno != EISREMOTE
#endif /* EISREMOTE */
				)
				adios(distfile, "unable to link %s to",
						altmsg);
			free(distfile);
			distfile = getcpy(m_mktemp2(NULL, invo_name,
					NULL, NULL));
			{
				int in, out;
				struct stat st;

				if ((in = open(altmsg, O_RDONLY)) == NOTOK)
					adios(altmsg, "unable to open");
				fstat(in, &st);
				if ((out = creat(distfile,
						(int) st.st_mode & 0777))
						== NOTOK)
					adios(distfile, "unable to write");
				cpydata(in, out, altmsg, distfile);
				close(in);
				close(out);
			}
		}
	} else {
		distfile = NULL;
	}

	if (altmsg == NULL || stat(altmsg, &st) == NOTOK) {
		st.st_mtime = 0;
		st.st_dev = 0;
		st.st_ino = 0;
	}
	if (pushsw)
		push();

	status = 0;
	vec[0] = mhbasename(postproc);
	closefds(3);

	for (msgnum = 0; msgnum < msgp; msgnum++) {
		switch (sendsbr(vec, vecp, msgs[msgnum], &st, 1)) {
		case DONE:
			done(++status);
		case NOTOK:
			status++;  /* fall */
		case OK:
			break;
		}
	}

	context_save();  /* save the context file */
	done(status);
	return 1;
}
